# MIT License
#
# Copyright (c) 2020 - Present nxtlo
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""A helper module for useful decorators and other stuff."""


from __future__ import annotations

__all__: typing.Sequence[str] = [
    "deprecated",
    "JsonDict",
    "JsonList",
    "Undefined",
    "Unknown",
]

import typing
import warnings
from functools import wraps

JsonDict = typing.Dict[str, typing.Any]
JsonList = typing.List[typing.Dict[str, typing.Any]]

# For stuff thats == ''
Undefined: str = "Undefined"

# For unknown stuff.
Unknown: typing.Optional[str] = ""


def deprecated(func) -> typing.Callable[[typing.Any], None]:
    """
    functions with this decorator will not work or is not implemented yet.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        warnings.warn("This function is no longer used or not fully implemented yet.")
        coro = func(*args, **kwargs)
        return coro

    return wrapper
